/**************/
/* hiiru.c    */

/* Tss tiedostossa olevat funktiot kutsuvat Microsoft-yhteensopivien   */
/* hiirien keskeytysfunktioita.                                          */



#include <dos.h>

#define VASEN 0
#define OIKEA 1



/*                                                                       */
/* Aliohjelma alustaa hiiren toimintaa varten. Funktio palauttaa -1:n    */
/* mikli onnistui, muuten nollan.                                       */
/*                                                                       */

int alustahiiri(void)
{
 struct REGPACK regs;

 regs.r_ax=0;
 intr(51,&regs);
 return regs.r_ax;
}




/*                                                                       */
/* Aliohjelma tekee hiirikursorin nkyvksi.                             */
/*                                                                       */

void naytahiirikursori(void)
{
 struct REGPACK regs;

 regs.r_ax=1;
 intr(51,&regs);
}




/*                                                                       */
/* Aliohjelma poistaa hiirikursorin kuvaruudulta. Tt funktiota on      */
/* kutsuttava aina, kun tulostetaan ruudulle jotakin, muuten tulostus    */
/* saattaa seota. HUOM. Jos tt on kutsuttu useammin kuin kerran ilman  */
/* ett olisi vlill kutsuttu funktiota naytahiirikursori, on sit      */
/* kutsuttava mys yht monta kertaa, jotta kursori saataisiin nkyville.*/
/*                                                                       */

void katkehiirikursori(void)
{
 struct REGPACK regs;

 regs.r_ax=2;
 intr(51,&regs);
}




/*                                                                       */
/* Aliohjelma lukee hiiren paikan ja nappien tilan.                      */
/* Palauttaa:                                                            */
/*   x       hiiren x-koordinaatti ruudulla (pikseleiss)                */
/*   y       hiiren y-koordinaatti ruudulla (pikseleiss)                */
/*   nappi1  1, mikli hiiren vasen nappi on alhaalla, muuten 0          */
/*   nappi2  1, mikli hiiren oikea nappi on alhaalla, muuten 0          */
/*                                                                       */

void hiirentila(int *x,int *y,int *nappi1,int *nappi2)
{
 struct REGPACK regs;

 regs.r_ax=3;
 intr(51,&regs);
 *x=regs.r_cx;
 *y=regs.r_dx;
 *nappi1=(regs.r_bx & 1);
 *nappi2=(regs.r_bx & 2) >> 1;
}




/*                                                                       */
/* Aliohjelma siirt hiiren kursorin haluttuun kohtaan kuvaruudulla.    */
/*                                                                       */

void siirrahiirikursori(int x,int y)
{
 struct REGPACK regs;

 regs.r_ax=4;
 regs.r_cx=x;
 regs.r_dx=y;
 intr(51,&regs);
}




/*                                                                       */
/* Aliohjelma antaa tietoa hiiren napin painamisesta.                    */
/* Parametri tutkittava ilmoittaa kumpaa nappia halutaan tutkia:         */
/* 0=vasen nappi,1=oikea nappi.                                          */
/* Palauttaa:                                                            */
/*   nappi1   1, mikli hiiren vasen nappi on kutsuttaessa alhaalla,     */
/*	      muuten 0                                                   */
/*   nappi2   1, mikli hiiren oikea nappi on kutsuttaessa alhaalla,     */
/*	      muuten 0                                                   */
/*   montako  ilmoittaa, montako kertaa tutkittavaa nappia on painettu   */
/*	      edellisen kutsun jlkeen.                                  */
/*   x        hiiren x-koordinaatti tutkittavan napin edellisen          */
/*	      painalluksen tapahtumahetkell.                            */
/*   y        hiiren y-koordinaatti tutkittavan napin edellisen          */
/*	      painalluksen tapahtumahetkell.                            */
/*                                                                       */

void hiirenpainallukset(int tutkittava,int *nappi1,int *nappi2,
			int *montako,int *x,int *y)
{
 struct REGPACK regs;

 regs.r_ax=5;
 regs.r_bx=tutkittava;
 intr(51,&regs);
 *x=regs.r_cx;
 *y=regs.r_dx;
 *nappi1=(regs.r_ax & 1);
 *nappi2=(regs.r_ax & 2) >> 1;
 *montako=regs.r_bx;
}




/*                                                                       */
/* Aliohjelma antaa tietoa hiiren napin vapauttamisesta.                 */
/* Parametri tutkittava ilmoittaa kumpaa nappia halutaan tutkia:         */
/* 0=vasen nappi,1=oikea nappi.                                          */
/* Palauttaa:                                                            */
/*   nappi1   1, mikli hiiren vasen nappi on kutsuttaessa alhaalla,     */
/*            muuten 0                                                   */
/*   nappi2   1, mikli hiiren oikea nappi on kutsuttaessa alhaalla,     */
/*	      muuten 0                                                   */
/*   montako  ilmoittaa, montako kertaa tutkitta nappi on vapautettu     */
/*	      edellisen kutsun jlkeen.                                  */
/*   x        hiiren x-koordinaatti tutkittavan napin edellisen          */
/*	      vapautuksen tapahtumahetkell.                             */
/*   y        hiiren y-koordinaatti tutkittavan napin edellisen          */
/*	      vapautuksen tapahtumahetkell.                             */
/*                                                                       */

void hiirenvapautukset(int tutkittava,int *nappi1,int *nappi2,
		       int *montako,int *x,int *y)
{
 struct REGPACK regs;

 regs.r_ax=6;
 regs.r_bx=tutkittava;
 intr(51,&regs);
 *x=regs.r_cx;
 *y=regs.r_dx;
 *nappi1=(regs.r_ax & 1);
 *nappi2=(regs.r_ax & 2) >> 1;
 *montako=regs.r_bx;
}




/*                                                                       */
/* Aliohjelma mr suorakaiteen muotoisen alueen, jolla hiirikursori   */
/* saa liikkua. Vasen on alueen vasemman laidan x-koordinaatti, yla on   */
/* alueen ylreunan y-koordinaatti, oikea on alueen oikean laidan        */
/* x-koordinaatti, ala on alueen alareunan y-koordinaatti.               */
/*                                                                       */

void hiirenalue(int vasen,int yla,int oikea,int ala)
{
 struct REGPACK regs;

 regs.r_ax=7;
 regs.r_cx=vasen;
 regs.r_dx=oikea;
 intr(51,&regs);
 regs.r_ax=8;
 regs.r_cx=yla;
 regs.r_dx=ala;
 intr(51,&regs);
}




/*                                                                       */
/* Aliohjelma mrittelee grafiikkatilassa toimivan hiirikursorin.       */
/*                                                                       */

void grhiirikursori(unsigned *kursori,int hotspotx,int hotspoty)
{
 struct REGPACK regs;

 regs.r_ax=9;
 regs.r_bx=hotspotx;
 regs.r_cx=hotspoty;
 regs.r_dx=FP_OFF(kursori);
 regs.r_es=FP_SEG(kursori);
 intr(51,&regs);
}




/*                                                                       */
/* Aliohjelma mr alueen, jolla hiirikursori poistetaan ruudulta,     */
/* jos se siirretn kyseiselle alueelle. Kun hiirikursori liikutetaan   */
/* pois tlt alueelta, se ei tule automaattisesti nkyviin, vaan on     */
/* kutsuttava aliohjelmaa naytahiirikursori.                             */
/*                                                                       */

void poista_alueeltakursori(int vasen,int yla,int oikea,int ala)
{
 struct REGPACK regs;

 regs.r_ax=16;
 regs.r_cx=vasen;
 regs.r_dx=yla;
 regs.r_si=oikea;
 regs.r_di=ala;
 intr(51,&regs);
}