/**************/
/* hiiru.h    */
/*************************************************************************/
/* Tss tiedostossa olevat funktiot kutsuvat Microsoft-yhteensopivien   */
/* hiirien keskeytysfunktioita.                                          */



#include <dos.h>

#define VASEN 0
#define OIKEA 1



/*                                                                       */
/* Aliohjelma alustaa hiiren toimintaa varten. Funktio palauttaa -1:n    */
/* mikli onnistui, muuten nollan.                                       */
/*                                                                       */

int alustahiiri(void);



/*                                                                       */
/* Aliohjelma tekee hiirikursorin nkyvksi.                             */
/*                                                                       */

void naytahiirikursori(void);




/*                                                                       */
/* Aliohjelma poistaa hiirikursorin kuvaruudulta. Tt funktiota on      */
/* kutsuttava aina, kun tulostetaan ruudulle jotakin, muuten tulostus    */
/* saattaa seota. HUOM. Jos tt on kutsuttu useammin kuin kerran ilman  */
/* ett olisi vlill kutsuttu funktiota naytahiirikursori, on sit      */
/* kutsuttava mys yht monta kertaa, jotta kursori saataisiin nkyville.*/
/*                                                                       */

void katkehiirikursori(void);




/*                                                                       */
/* Aliohjelma lukee hiiren paikan ja nappien tilan.                      */
/* Palauttaa:                                                            */
/*   x       hiiren x-koordinaatti ruudulla (pikseleiss)                */
/*   y       hiiren y-koordinaatti ruudulla (pikseleiss)                */
/*   nappi1  1, mikli hiiren vasen nappi on alhaalla, muuten 0          */
/*   nappi2  1, mikli hiiren oikea nappi on alhaalla, muuten 0          */
/*                                                                       */

void hiirentila(int *x,int *y,int *nappi1,int *nappi2);




/*                                                                       */
/* Aliohjelma siirt hiiren kursorin haluttuun kohtaan kuvaruudulla.    */
/*                                                                       */

void siirrahiirikursori(int x,int y);




/*                                                                       */
/* Aliohjelma antaa tietoa hiiren napin painamisesta.                    */
/* Parametri tutkittava ilmoittaa kumpaa nappia halutaan tutkia:         */
/* 0=vasen nappi,1=oikea nappi.                                          */
/* Palauttaa:                                                            */
/*   nappi1   1, mikli hiiren vasen nappi on kutsuttaessa alhaalla,     */
/*	      muuten 0                                                   */
/*   nappi2   1, mikli hiiren oikea nappi on kutsuttaessa alhaalla,     */
/*	      muuten 0                                                   */
/*   montako  ilmoittaa, montako kertaa tutkittavaa nappia on painettu   */
/*	      edellisen kutsun jlkeen.                                  */
/*   x        hiiren x-koordinaatti tutkittavan napin edellisen          */
/*	      painalluksen tapahtumahetkell.                            */
/*   y        hiiren y-koordinaatti tutkittavan napin edellisen          */
/*	      painalluksen tapahtumahetkell.                            */
/*                                                                       */

void hiirenpainallukset(int tutkittava,int *nappi1,int *nappi2,
			int *montako,int *x,int *y);




/*                                                                       */
/* Aliohjelma antaa tietoa hiiren napin vapauttamisesta.                 */
/* Parametri tutkittava ilmoittaa kumpaa nappia halutaan tutkia:         */
/* 0=vasen nappi,1=oikea nappi.                                          */
/* Palauttaa:                                                            */
/*   nappi1   1, mikli hiiren vasen nappi on kutsuttaessa alhaalla,     */
/*            muuten 0                                                   */
/*   nappi2   1, mikli hiiren oikea nappi on kutsuttaessa alhaalla,     */
/*	      muuten 0                                                   */
/*   montako  ilmoittaa, montako kertaa tutkitta nappi on vapautettu     */
/*	      edellisen kutsun jlkeen.                                  */
/*   x        hiiren x-koordinaatti tutkittavan napin edellisen          */
/*	      vapautuksen tapahtumahetkell.                             */
/*   y        hiiren y-koordinaatti tutkittavan napin edellisen          */
/*	      vapautuksen tapahtumahetkell.                             */
/*                                                                       */

void hiirenvapautukset(int tutkittava,int *nappi1,int *nappi2,
		       int *montako,int *x,int *y);




/*                                                                       */
/* Aliohjelma mr suorakaiteen muotoisen alueen, jolla hiirikursori   */
/* saa liikkua. Vasen on alueen vasemman laidan x-koordinaatti, yla on   */
/* alueen ylreunan y-koordinaatti, oikea on alueen oikean laidan        */
/* x-koordinaatti, ala on alueen alareunan y-koordinaatti.               */
/*                                                                       */

void hiirenalue(int vasen,int yla,int oikea,int ala);




/*                                                                       */
/* Aliohjelma mrittelee grafiikkatilassa toimivan hiirikursorin.       */
/*                                                                       */

void grhiirikursori(unsigned *kursori,int hotspotx,int hotspoty);




/*                                                                       */
/* Aliohjelma mr alueen, jolla hiirikursori poistetaan ruudulta,     */
/* jos se siirretn kyseiselle alueelle. Kun hiirikursori liikutetaan   */
/* pois tlt alueelta, se ei tule automaattisesti nkyviin, vaan on     */
/* kutsuttava aliohjelmaa naytahiirikursori.                             */
/*                                                                       */

void poista_alueeltakursori(int vasen,int yla,int oikea,int ala);